@@ -10,6 +10,7 @@ from astropy import units as apu
 import numpy as np
 
 from . import cyimt
+from .helper import _Qinv
 from .. import conversions as cnv
 from .. import utils
 # import ipdb
@@ -39,47 +40,6 @@ UE_MIN_MAX_DISTANCES = {
 # ranged_quantity_input fails)
 
 
-def _Qinv(x):
-    # Note, this is *not* identical to cyprop._I_helper
-    # only good between 1.e-6 and 0.5
-    # See R-Rec P.1546
-
-    x = np.atleast_1d(x).copy()
-    mask = x > 0.5
-    x[mask] = 1 - x[mask]
-
-    T = np.sqrt(-2 * np.log(x))
-    Z = (
-        (
-            ((0.010328 * T + 0.802853) * T) + 2.515516698
-            ) /
-        (
-            ((0.001308 * T + 0.189269) * T + 1.432788) * T + 1.
-            )
-        )
-
-    Q = T - Z
-    Q[mask] *= -1
-    return Q
-
-
-# def Qinv(x):
-#     # larger x range than the approximation given in P.1546?
-#     # definitely much slower
-
-#     from scipy.stats import norm as qnorm
-
-#     x = np.atleast_1d(x).copy()
-
-#     mask = x > 0.5
-#     x[mask] = 1 - x[mask]
-
-#     Q = -qnorm.ppf(x, 0)
-#     Q[mask] *= -1
-
-#     return Q
-
-
 def _clutter_imt(
         freq,
         dist,
@@ -571,7 +531,6 @@ def P_pusch(P_cmax, M_pusch, P_0_pusch, alpha, PL):
         Note: Antenna gains should be included, so this is rather the
         coupling loss than the path loss, unlike what is stated in
         `ITU-R Rec. M.2101-0 <https://www.itu.int/rec/R-REC-M.2101/en>`_.
-        ``
 
     Returns
     -------
