@@ -12,6 +12,7 @@ import numpy as np
 from . import cyprop
 from . import heightprofile
 from . import helper
+from . import srtm
 from .. import conversions as cnv
 from .. import utils
 # import ipdb
@@ -21,7 +22,7 @@ __all__ = [
     'PathProp',
     'loss_freespace', 'loss_troposcatter', 'loss_ducting',
     'loss_diffraction', 'loss_complete',
-    'clutter_correction', 'clutter_imt',
+    'clutter_correction',
     'height_map_data', 'atten_map_fast',
     'height_path_data', 'height_path_data_generic', 'atten_path_fast',
     'losses_complete',
@@ -97,6 +98,16 @@ class PathProp(cyprop._PathProp):
     hprof_backbearing : `~astropy.units.Quantity`, optional
         Back-bearing of the height profile path.
         (default: query `~pycraf.pathprof.srtm_height_profile`)
+    generic_heights : bool
+        If `generic_heights` is set to True, heights will be set to zero.
+        This can be useful for generic (aka flat-Earth) computations.
+        The option is only meaningful, if the hprof_xxx parameters are set
+        to `None` (which means automatic querying of the profiles).
+        (Default: False)
+    base_water_density : `~astropy.units.Quantity`, optional
+        For atmospheric attenuation, the water vapor content plays a role.
+        In Rec. ITU-R P.452, Eq. (9a), the water content is variable (depending on the fraction of the path over the water). However, the base level is set to :math:`7.5~\\mathrm{g}/\\mathrm{m}^3`. For extraordinarily dry places, which are often used for radio astronomy, this value can be too high.
+        (Default: 7.5 g / m^3)
 
     Returns
     -------
@@ -167,6 +178,7 @@ class PathProp(cyprop._PathProp):
         hprof_heights=(None, None, apu.m),
         hprof_bearing=(None, None, apu.deg),
         hprof_backbearing=(None, None, apu.deg),
+        base_water_density=(0.5, 100, apu.g / apu.m ** 3),
         strip_input_units=True, allow_none=True, output_unit=None
         )
     def __init__(
@@ -190,6 +202,8 @@ class PathProp(cyprop._PathProp):
             # override if you don't want builtin method:
             hprof_dists=None, hprof_heights=None,
             hprof_bearing=None, hprof_backbearing=None,
+            generic_heights=False,
+            base_water_density=7.5 * apu.g / apu.m ** 3
             ):
 
         super().__init__(
@@ -212,6 +226,8 @@ class PathProp(cyprop._PathProp):
             hprof_heights=hprof_heights,
             hprof_bearing=hprof_bearing,
             hprof_backbearing=hprof_backbearing,
+            generic_heights=generic_heights,
+            base_water_density=base_water_density,
             )
 
         self.__params = list(cyprop.PARAMETERS_BASIC)  # make a copy
@@ -463,130 +479,6 @@ def clutter_correction(
     return cyprop.clutter_correction_cython(h_g, zone, freq)
 
 
-def _clutter_imt(
-        freq,
-        dist,
-        location_percent,
-        num_end_points=1,
-        ):
-
-    assert num_end_points in [1, 2]
-
-    def Qinv(x):
-        # Note, this is *not* identical to cyprop._I_helper
-        # only good between 1.e-6 and 0.5
-        # See R-Rec P.1546
-
-        x = np.atleast_1d(x).copy()
-        mask = x > 0.5
-        x[mask] = 1 - x[mask]
-
-        T = np.sqrt(-2 * np.log(x))
-        Z = (
-            (
-                ((0.010328 * T + 0.802853) * T) + 2.515516698
-                ) /
-            (
-                ((0.001308 * T + 0.189269) * T + 1.432788) * T + 1.
-                )
-            )
-
-        Q = T - Z
-        Q[mask] *= -1
-        return Q
-
-    # def Qinv(x):
-    #     # larger x range than the approximation given in P.1546?
-    #     # definitely much slower
-
-    #     from scipy.stats import norm as qnorm
-
-    #     x = np.atleast_1d(x).copy()
-
-    #     mask = x > 0.5
-    #     x[mask] = 1 - x[mask]
-
-    #     Q = -qnorm.ppf(x, 0)
-    #     Q[mask] *= -1
-
-    #     return Q
-
-    L_l = 23.5 + 9.6 * np.log10(freq)
-    L_s = 32.98 + 23.9 * np.log10(dist) + 3.0 * np.log10(freq)
-
-    L_clutter = -5 * np.log10(
-        np.power(10, -0.2 * L_l) + np.power(10, -0.2 * L_s)
-        ) - 6 * Qinv(location_percent / 100.)
-
-    if num_end_points == 2:
-        L_clutter *= 2
-
-    return L_clutter
-
-
-@utils.ranged_quantity_input(
-    freq=(2, 67, apu.GHz),
-    dist=(0.25, None, apu.km),
-    location_percent=(0, 100, apu.percent),
-    strip_input_units=True, output_unit=cnv.dB
-    )
-def clutter_imt(
-        freq,
-        dist,
-        location_percent,
-        num_end_points=1,
-        ):
-    '''
-    Calculate the Clutter loss according to IMT.CLUTTER document (method 2).
-
-    Parameters
-    ----------
-    freq : `~astropy.units.Quantity`
-        Frequency of radiation [GHz]
-    dist : `~astropy.units.Quantity`
-        Distance between Tx/Rx antennas [km]
-
-        Minimal distance must be 0.25 km (single endpoint clutter) or 1 km
-        (if both endpoints are to be corrected for clutter)
-    location_percent : `~astropy.units.Quantity`
-        Percentage of locations for which the clutter loss `L_clutter`
-        (calculated with this function) will not be exceeded [%]
-    num_end_points : int, optional
-        number of endpoints affected by clutter, allowed values: 1, 2
-
-    Returns
-    -------
-    L_clutter : `~astropy.units.Quantity`
-        Clutter loss [dB]
-
-    Notes
-    -----
-    - The algorithm is independent of effective antenna height (w.r.t.
-      clutter height), i.e., it doesn't distinguish between terminals which
-      are close to the ground and those closer to the top of the building.
-      However, the model is only appropriate if the terminal is "in the
-      clutter", below the rooftops.
-    - The result of this function is to be understood as a cumulative
-      value. For example, if `location_percent = 2%`, it means that for
-      2% of all possible locations, the clutter loss will not exceed the
-      returned `L_clutter` value, for the remaining 98% of locations it
-      will therefore be lower than `L_clutter`. The smaller `location_percent`,
-      the smaller the returned `L_clutter`, i.e., low clutter attenuations
-      are more unlikely.
-    - This model was proposed by ITU study group SG 3 to replace
-      `~pycraf.pathprof.clutter_correction` for IMT 5G studies (especially,
-      at higher frequencies, where multipath effects play a role in
-      urban and suburban areas).
-    '''
-
-    return _clutter_imt(
-        freq,
-        dist,
-        location_percent,
-        num_end_points=num_end_points,
-        )
-
-
 # TODO: do we want to convert output dictionary arrays to quantities?
 @utils.ranged_quantity_input(
     lon_t=(-180, 180, apu.deg),
@@ -610,6 +502,7 @@ def height_map_data(
         d_tm=None, d_lm=None,
         d_ct=None, d_cr=None,
         omega_percent=0 * apu.percent,
+        cache_path=None, clobber=False,
         ):
 
     '''
@@ -668,6 +561,17 @@ def height_map_data(
     omega_percent : `~astropy.units.Quantity`, optional
         Fraction of the path over water [%] (see Table 3)
         (default: 0%)
+    cache_path : str, optional
+        If set, the `joblib package
+        <https://joblib.readthedocs.io/en/latest/>`_ is used to cache
+        results provided by this function on disk, such that future queries
+        are executed much faster. If set to `None`, no caching is performed.
+        (default: None)
+    clobber : bool, optional
+        If set to `True` and caching is active re-compute the result even
+        if an older result is found in cache. This is useful, when something
+        has change with the underlying terrain data, e.g., new tiles were
+        downloaded. (default: `False`)
 
     Returns
     -------
@@ -783,17 +687,56 @@ def height_map_data(
       For details see :ref:`working_with_srtm`.
     '''
 
-    return cyprop.height_map_data_cython(
-        lon_t, lat_t,
-        map_size_lon, map_size_lat,
+    args = lon_t, lat_t, map_size_lon, map_size_lat
+    kwargs = dict(
         map_resolution=map_resolution,
         do_cos_delta=1 if do_cos_delta else 0,
-        zone_t=zone_t, zone_r=zone_r,
+        zone_t=int(zone_t), zone_r=int(zone_r),  # needed for joblib
         d_tm=d_tm, d_lm=d_lm,
         d_ct=d_ct, d_cr=d_cr,
         omega=omega_percent,
         )
 
+    joblib_available = True
+    try:
+        import joblib
+        memory = joblib.Memory(cache_path, compress=5, verbose=False)
+    except ImportError:
+        joblib_available = False
+
+    if cache_path and joblib_available:
+
+        def wrapped(*args, **kwargs):
+            @memory.cache
+            def f(tile_path, interp, spline_opts, *args, **kwargs):
+                return cyprop.height_map_data_cython(*args, **kwargs)
+
+            if clobber:
+                result = f.call_and_shelve(
+                    srtm.SrtmConf.srtm_dir,
+                    srtm.SrtmConf.interp,
+                    srtm.SrtmConf.spline_opts,
+                    *args, **kwargs
+                    )
+                try:
+                    result.clear()
+                except KeyError:
+                    pass
+
+            return f(
+                srtm.SrtmConf.srtm_dir,
+                srtm.SrtmConf.interp,
+                srtm.SrtmConf.spline_opts,
+                *args, **kwargs
+                )
+
+    elif cache_path and not joblib_available:
+        raise ImportError('"joblib" package must be installed for caching')
+    else:
+        wrapped = cyprop.height_map_data_cython
+
+    return wrapped(*args, **kwargs)
+
 
 @utils.ranged_quantity_input(
     freq=(0.1, 100, apu.GHz),
@@ -802,6 +745,7 @@ def height_map_data(
     h_tg=(None, None, apu.m),
     h_rg=(None, None, apu.m),
     timepercent=(0, 50, apu.percent),
+    base_water_density=(0.5, 100, apu.g / apu.m ** 3),
     strip_input_units=True,
     )
 def atten_map_fast(
@@ -813,6 +757,7 @@ def atten_map_fast(
         hprof_data,  # dict_like
         polarization=0,
         version=16,
+        base_water_density=7.5 * apu.g / apu.m ** 3,
         ):
     '''
     Calculate attenuation maps using a fast method.
@@ -838,6 +783,10 @@ def atten_map_fast(
         Allowed values are: 0 - horizontal, 1 - vertical
     version : int, optional
         ITU-R Rec. P.452 version. Allowed values are: 14, 16
+    base_water_density : `~astropy.units.Quantity`, optional
+        For atmospheric attenuation, the water vapor content plays a role.
+        In Rec. ITU-R P.452, Eq. (9a), the water content is variable (depending on the fraction of the path over the water). However, the base level is set to :math:`7.5~\\mathrm{g}/\\mathrm{m}^3`. For extraordinarily dry places, which are often used for radio astronomy, this value can be too high.
+        (Default: 7.5 g / m^3)
 
     Returns
     -------
@@ -889,6 +838,7 @@ def atten_map_fast(
         hprof_data,  # dict_like
         polarization=polarization,
         version=version,
+        base_water_density=base_water_density,
         )
 
     return {
@@ -1234,6 +1184,7 @@ def height_path_data_generic(
     h_tg=(None, None, apu.m),
     h_rg=(None, None, apu.m),
     timepercent=(0, 50, apu.percent),
+    base_water_density=(0.5, 100, apu.g / apu.m ** 3),
     strip_input_units=True,
     )
 def atten_path_fast(
@@ -1245,6 +1196,7 @@ def atten_path_fast(
         hprof_data,  # dict_like
         polarization=0,
         version=16,
+        base_water_density=7.5 * apu.g / apu.m ** 3,
         ):
     '''
     Calculate attenuation along a path using a parallelized method.
@@ -1270,6 +1222,10 @@ def atten_path_fast(
         Allowed values are: 0 - horizontal, 1 - vertical
     version : int, optional
         ITU-R Rec. P.452 version. Allowed values are: 14, 16
+    base_water_density : `~astropy.units.Quantity`, optional
+        For atmospheric attenuation, the water vapor content plays a role.
+        In Rec. ITU-R P.452, Eq. (9a), the water content is variable (depending on the fraction of the path over the water). However, the base level is set to :math:`7.5~\\mathrm{g}/\\mathrm{m}^3`. For extraordinarily dry places, which are often used for radio astronomy, this value can be too high.
+        (Default: 7.5 g / m^3)
 
     Returns
     -------
@@ -1373,6 +1329,7 @@ def atten_path_fast(
         hprof_data,  # dict_like
         polarization=polarization,
         version=version,
+        base_water_density=base_water_density,
         )
 
     return {
@@ -1415,6 +1372,7 @@ def atten_path_fast(
     hprof_heights=(None, None, apu.m),
     hprof_bearing=(None, None, apu.deg),
     hprof_backbearing=(None, None, apu.deg),
+    base_water_density=(0.5, 100, apu.g / apu.m ** 3),
     strip_input_units=True, allow_none=True, output_unit=None
     )
 def losses_complete(
@@ -1438,6 +1396,8 @@ def losses_complete(
         # override if you don't want builtin method:
         hprof_dists=None, hprof_heights=None,
         hprof_bearing=None, hprof_backbearing=None,
+        generic_heights=False,
+        base_water_density=7.5 * apu.g / apu.m ** 3,
         ):
     '''
     Calculate propagation losses for a fixed path using a parallelized method.
@@ -1512,6 +1472,16 @@ def losses_complete(
     hprof_backbearing : `~astropy.units.Quantity`, optional
         Back-bearing of the height profile path.
         (default: query `~pycraf.pathprof.srtm_height_profile`)
+    generic_heights : bool
+        If `generic_heights` is set to True, heights will be set to zero.
+        This can be useful for generic (aka flat-Earth) computations.
+        The option is only meaningful, if the hprof_xxx parameters are set
+        to `None` (which means automatic querying of the profiles).
+        (Default: False)
+    base_water_density : `~astropy.units.Quantity`, optional
+        For atmospheric attenuation, the water vapor content plays a role.
+        In Rec. ITU-R P.452, Eq. (9a), the water content is variable (depending on the fraction of the path over the water). However, the base level is set to :math:`7.5~\\mathrm{g}/\\mathrm{m}^3`. For extraordinarily dry places, which are often used for radio astronomy, this value can be too high.
+        (Default: 7.5 g / m^3)
 
     Returns
     -------
@@ -1627,6 +1597,8 @@ def losses_complete(
         hprof_heights=hprof_heights,
         hprof_bearing=hprof_bearing,
         hprof_backbearing=hprof_backbearing,
+        generic_heights=generic_heights,
+        base_water_density=base_water_density,
         )
     return {
         'L_b0p': res[0] * cnv.dB,
