@@ -40,16 +40,17 @@ class _MultiMeta(type):
                 )
 
     def __repr__(cls):
-        if hasattr(cls, '__repr__'):
-            return getattr(cls, '__repr__')()
-        else:
-            return super().__repr__()
+        # this leads to a strange sphinx error...
+        # if hasattr(cls, '__repr__'):
+        #     return getattr(cls, '__repr__')()
+        # else:
+        return super().__repr__()
 
     def __str__(cls):
-        if hasattr(cls, '__str__'):
-            return getattr(cls, '__str__')()
-        else:
-            return super().__repr__()
+        # if hasattr(cls, '__str__'):
+        #     return getattr(cls, '__str__')()
+        # else:
+        return super().__repr__()
 
 
 class MultiState(object, metaclass=_MultiMeta):
